#!/usr/local/bin/perl -s

use Image::Magick;
# use Data::Dumper;

$| = 1;

($ww,$hh) = (500,500) if $big and not defined $ww;
($ww,$hh) = (100,100) if not defined $ww;

print "ww = $ww, hh = $hh\n";
$suffix = $big? '' : '_t';

$avg = [];
foreach my $y (0..$hh-1)
{
  $avg->[$y] = [];
  foreach my $x (0..$ww-1)
  {
    $avg->[$y]->[$x] = [0,0,0];
  }
}

$rname = shift;

$incfile = $rname . '.ph' if not $rname =~ /\.ph/;
$incfile = 'photolists/' . $incfile if not -e $incfile and -e "photolists/$incfile";

require $incfile;

my $totImages = 0;
$n = 0;
foreach my $photo (@books)
{
  my $ifile = MakeLocalPath($photo,$suffix);
  # print $ifile . "\n";
  next if not -e $ifile;
  
	my $image = Image::Magick->new;
	my $err = $image->Read($ifile);
	print $err if $err;
	my ($w,$h) = $image->Get('width','height');
  my $aspect = $ww/$hh;
  if ($w/$h < $aspect) {
      # horizontal strip
      my $nh = $w / $aspect;
      $image->Crop(width=>$w, height=>$nh, 'x'=>0, 'y'=>($h-$nh)/2);
  }
  elsif ($w/$h > $aspect) {
      my $nw = $h * $aspect;
      $image->Crop(width=>$nw, height=>$h, 'y'=>0, 'x'=>($w-$nw)/2);
  }
	$image->Resize(geometry=>"${ww}x${hh}");
	my ($w,$h) = ($ww,$hh);

  my @pix = $image->GetPixels('x'=>0,'y'=>0,'width'=>$w,'height'=>$h,'normalize'=>1);
  my $j = 0;

	foreach $y (0..$h-1) 
	{
		foreach $x (0..$w-1)
		{
        $avg->[$y]->[$x]->[0] += $pix[$j++];     
        $avg->[$y]->[$x]->[1] += $pix[$j++];     
        $avg->[$y]->[$x]->[2] += $pix[$j++];     
		}
	}
  $totImages++;
  undef $image;
  $n++;
  print "\r$n" if $n % 100 == 0;
  last if $limit and $n >= $limit;
}

print "\n\nFinding min and max\n";
$min = $totImages;
$max = 0;
foreach my $y (0..$hh-1)
{
  foreach my $x (0..$ww-1)
  {
    $min = $avg->[$y]->[$x]->[0] if $avg->[$y]->[$x]->[0] < $min;
    $min = $avg->[$y]->[$x]->[1] if $avg->[$y]->[$x]->[1] < $min;
    $min = $avg->[$y]->[$x]->[2] if $avg->[$y]->[$x]->[2] < $min;

    $max = $avg->[$y]->[$x]->[0] if $avg->[$y]->[$x]->[0] > $max;
    $max = $avg->[$y]->[$x]->[1] if $avg->[$y]->[$x]->[1] > $max;
    $max = $avg->[$y]->[$x]->[2] if $avg->[$y]->[$x]->[2] > $max;
  }
}
$range = ($max-$min);

# make nice normalized image...
my $image2 = Image::Magick->new;
$image2->Set(size=>"${ww}x${hh}");
$image2->Read('xc:#FFFFFF');
foreach my $y (0..$hh-1)
{
  foreach my $x (0..$ww-1)
  {
	  $image2->SetPixel('x'=>$x,'y'=>$y,'width'=>1,'height'=>1,'normalize'=>1, 'color' => [($avg->[$y]->[$x]->[0]-$min)/$range,($avg->[$y]->[$x]->[1]-$min)/$range,($avg->[$y]->[$x]->[2]-$min)/$range, 1]);
  }
}
$image2->Write($rname . "_normx_${limit}.png");
undef $image2;

print "\n\nAveraging $totImages images\n";
foreach my $y (0..$hh-1)
{
  foreach my $x (0..$ww-1)
  {
    $avg->[$y]->[$x]->[0] /= $totImages;
    $avg->[$y]->[$x]->[1] /= $totImages;
    $avg->[$y]->[$x]->[2] /= $totImages;
  }
}

$outimage = Image::Magick->new;
$outimage->Set(size=>"${mwidth}x${mheight}");


my $image = Image::Magick->new;
$image->Set(size=>"${ww}x${hh}");
$image->Read('xc:#FFFFFF');

print "Compositing...\n";
foreach my $y (0..$hh-1)
{
  foreach my $x (0..$ww-1)
  {
	  $image->SetPixel('x'=>$x,'y'=>$y,'width'=>1,'height'=>1,'normalize'=>1, 'color' => [$avg->[$y]->[$x]->[0],$avg->[$y]->[$x]->[1],$avg->[$y]->[$x]->[2], 1]);
  }
}

print "Writing\n";
$image->Write($rname . "_avg.png");

# $image->Normalize();
#
# $image->Write($rname . "_norm.png");

exit;

sub MakeFlickrPath($$)
{
	my ($photo, $suffix) = @_;
	return sprintf "http://farm%s.static.flickr.com/%d/%s_%s%s.jpg", 
	            $photo->{farm},
				$photo->{server},
				$photo->{id}, 
				$photo->{secret}, $suffix;
}

sub MakeLocalPath($$)
{
	my ($photo, $suffix) = @_;
	return MakeDirName($photo->{id}) . "$photo->{id}$suffix.jpg";
}

sub MakeDirName($)
{
	my ($id) = @_;
	return sprintf 'flickrcache/%03d/%03d/', int($id/1000000)%1000, int($id/1000)%1000;
}

sub BuildDirs($)
{
  my ($lname) = @_;
  my @dirs = split /\//, $lname;
  $ldir = '';
  @dirs = splice(@dirs, 0, scalar(@dirs)-1);
  foreach (@dirs)
  {
    last if /\.jpg/i;
    $ldir .= '/' if $ldir ne '';
    $ldir .= $_;
    if (!(-e $ldir))
    {
      # print "Making $ldir ...\n";
      `mkdir $ldir`;
    }
  }
}
