#!/usr/local/bin/perl -s

use LWP::Simple  qw($ua get);
use Image::Magick;
use Data::Dumper;


$ua->timeout(10);
$Data::Dumper::Terse = 1;  # avoids $VAR1 = * ; in dumper output
$Data::Dumper::Indent = $verbose? 1 : 0;  # more concise output

$tag = shift;
die "Please provide a tag\n" if !$tag;
$tag =~ s/\.ph//;

require "$tag.ph";

$suffix = $big? '' : '_t';

foreach $p (@books)
{
  my $photo = $p;
  $url = MakeFlickrPath($photo, $suffix);
  $fname = MakeLocalPath($photo, $suffix);

  if (!(-e "$fname"))
  {
    print "Getting $url...\n";
    foreach (0..2)
    {
      $pimg = get $url;
      last if defined $pimg && $pimg ne '';
      print "Retry...\n";
      sleep 1;
    }
    if ($pimg) {
	  BuildDirs($fname);
      open (OFILE, ">$fname") || die ("can't open $fname\n");
      binmode OFILE;
      print OFILE $pimg;
      close OFILE;
    }
  }

  my $image = Image::Magick->new;
  $err = $image->Read("$fname");
  if ("$err") {
    warn "Warning: $err";
    next;
  }
  ($w,$h) = $image->Get('width','height');
  $x = $image->Resize(geometry=>'1x1');
  ($r,$g,$b) = $image->GetPixels(x=>0,y=>0,width=>1,height=>1,normalize=>1);
  ($p->{w}, $p->{h}, $p->{r}, $p->{g}, $p->{b}) = ($w,$h,$r,$g,$b);
  ++$nbrSampled;
}

open (OFILE, ">$tag.ph");
print OFILE "# prepared by sampleThumbs.pl\n";
print OFILE "\@books = (\n";
$n = 0;
foreach $book (@books)
{
  printf "Skipping %d x %d\n", $book->{w}, $book->{h} if !$book->{w};
  next if !$book->{w};
  print OFILE ($n++? ",\n" : "") . Dumper($book);
}
print OFILE "\n);\n1;\n";
close OFILE;

sub MakeFlickrPath($$)
{
	my ($photo, $suffix) = @_;
	return sprintf "http://farm%s.static.flickr.com/%d/%s_%s%s.jpg", 
	            $photo->{farm},
				$photo->{server},
				$photo->{id}, 
				$photo->{secret}, $suffix;
}

sub MakeLocalPath($$)
{
	my ($photo, $suffix) = @_;
	return MakeDirName($photo->{id}) . "$photo->{id}$suffix.jpg";
}

sub MakeDirName($)
{
	my ($id) = @_;
	return sprintf 'flickrcache/%03d/%03d/', int($id/1000000)%1000, int($id/1000)%1000;
}

sub BuildDirs($)
{
  my ($lname) = @_;
  my @dirs = split /\//, $lname;
  $ldir = '';
  @dirs = splice(@dirs, 0, scalar(@dirs)-1);
  foreach (@dirs)
  {
    last if /\.jpg/i;
    $ldir .= '/' if $ldir ne '';
    $ldir .= $_;
    if (!(-e $ldir))
    {
      # print "Making $ldir ...\n";
      `mkdir $ldir`;
    }
  }
}
